/*
 * TopMultiviewElement.java
 * 
 * 
 * Copyright (C) 2008 Markus Kilås
 *  
 * This file is part of TBN NetBeans Module.
 *
 * TBN NetBeans Module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TBN NetBeans Module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TBN NetBeans Module.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package com.markuspage.markus.tbn.module1;

import java.util.Arrays;
import javax.swing.Action;
import javax.swing.JComponent;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.netbeans.core.spi.multiview.MultiViewFactory;
import org.netbeans.modules.xml.multiview.AbstractMultiViewElement;
import org.netbeans.modules.xml.multiview.XmlMultiViewDataObject;
import org.openide.actions.FileSystemAction;
import org.openide.awt.UndoRedo;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;
import org.openide.util.lookup.ProxyLookup;

/**
 *
 * @author markus
 */
public class TopMultiviewElement implements MultiViewElement {

    private XmlMultiViewDataObject dObj;
    protected transient MultiViewElementCallback callback;
    private AbstractTopMultiViewPanel panel;

    public TopMultiviewElement(XmlMultiViewDataObject dObj, AbstractTopMultiViewPanel panel) {
        this.dObj = dObj;
        this.panel = panel;
    }

    public JComponent getVisualRepresentation() {
        return panel;
    }

    public JComponent getToolbarRepresentation() {
        return panel.getToolbarComponent();
    }

    public Action[] getActions() {
        Action[] actions = callback.createDefaultActions();
        SystemAction fsAction = SystemAction.get(FileSystemAction.class);
        if(!Arrays.asList(actions).contains(fsAction)) {
            Action[] newActions = new Action[actions.length + 1];
            System.arraycopy(actions, 0, newActions, 0, actions.length);
            newActions[actions.length] = fsAction;
            actions = newActions;
        }
        return actions;
    }

    public Lookup getLookup() {
        return new ProxyLookup(new org.openide.util.Lookup[]{
            dObj.getNodeDelegate().getLookup()
        });
    }

    public void componentOpened() {
        System.out.println("componentOpened");
    }

    public void componentClosed() {
        System.out.println("componentClosed");
    }

    public void componentShowing() {
        System.out.println("componentShowing");
        panel.reload();
    }

    public void componentHidden() {
        System.out.println("componentHidden");
    }

    public void componentActivated() {
        System.out.println("componentActivated");
    }

    public void componentDeactivated() {
        System.out.println("componentDeactivated");
    }

    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
        /*if(dObj!=null) {
            XmlMultiViewEditorSupport support = dObj.getEditorSupport();
            if(support!=null) {
                support.setMVTC(callback.getTopComponent());
                support.updateDisplayName();
            }
        }*/
    }

    public CloseOperationState canCloseElement() {
        if(dObj == null) {
            return CloseOperationState.STATE_OK;
        } else if(!dObj.canClose()) {
            return MultiViewFactory.createUnsafeCloseState(NbBundle.getMessage(AbstractMultiViewElement.class,
                    "LBL_DataObjectModified"), null, null);
        } else {
            return CloseOperationState.STATE_OK;
        }
    }
}

