/*
 * ADLToolbarMVElement.java
 * 
 * 
 * Copyright (C) 2008 Markus Kilås
 *  
 * This file is part of TBN NetBeans Module.
 *
 * TBN NetBeans Module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TBN NetBeans Module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TBN NetBeans Module.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package com.markuspage.markus.tbn.module1.adl;

import com.markuspage.markus.tbn.module1.adl.model.Binding;
import com.markuspage.markus.tbn.module1.adl.model.Channel;
import com.markuspage.markus.tbn.module1.adl.model.Component;
import com.markuspage.markus.tbn.module1.adl.model.Configuration;
import com.markuspage.markus.tbn.module1.adl.model.Factory;
import com.markuspage.markus.tbn.module1.adl.model.StartChannels;
import com.markuspage.markus.tbn.module1.adl.model.StartComponents;
import com.markuspage.markus.tbn.module1.adl.model.Subscription;
import org.netbeans.modules.xml.multiview.Error.ErrorLocation;
import org.netbeans.modules.xml.multiview.ToolBarMultiViewElement;
import org.netbeans.modules.xml.multiview.ui.SectionContainer;
import org.netbeans.modules.xml.multiview.ui.SectionContainerNode;
import org.netbeans.modules.xml.multiview.ui.SectionPanel;
import org.netbeans.modules.xml.multiview.ui.SectionView;
import org.netbeans.modules.xml.multiview.ui.ToolBarDesignEditor;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;

/**
 *
 * @author markus
 */
public class ADLToolBarMVElement extends ToolBarMultiViewElement {

    private ToolBarDesignEditor comp;
    private SectionView view;
    private ADLDataObject dataObject;
    private PanelFactory factory;

    public ADLToolBarMVElement(ADLDataObject dObj) {
        super(dObj);
        this.dataObject = dObj;
        comp = new ToolBarDesignEditor();
        factory = new PanelFactory(comp, dObj);
        setVisualEditor(comp);
    }

    @Override
    public SectionView getSectionView() {
        return view;
    }

    @Override
    public void componentShowing() {
        super.componentShowing();
        view = new ADLView(dataObject);
        comp.setContentView(view);
        try {
            view.openPanel(dataObject.getConfiguration());
        } catch (java.io.IOException ex) {
        }
        view.checkValidity();
    }

    private class ADLView extends SectionView {

        ADLView(ADLDataObject dObj) {
            super(factory);

            Children rootChildren = new Children.Array();
            Node root = new AbstractNode(rootChildren);
            try {
                Configuration book = dObj.getConfiguration();
                Node bookNode = new ConfigurationNode(book);

                // ADL Factories
                {
                    Factory[] chapters = book.getFactories().getFactory();
                    Node[] chapterNode = new Node[chapters.length];
                    Children ch = new Children.Array();
                    for (int i = 0; i < chapters.length; i++) {
                        chapterNode[i] = new FactoryNode(chapters[i]);
                    }
                    ch.add(chapterNode);
                    Node chaptersNode = new SectionContainerNode(ch);
                    chaptersNode.setDisplayName("Factories");
                    rootChildren.add(new Node[]{bookNode, chaptersNode});
                    // add panels
                    addSection(new SectionPanel(this, bookNode, book)); //NOI18N

                    SectionContainer chaptersCont = new SectionContainer(this, chaptersNode, "Factories");
                    //jspPGCont.setHeaderActions(new javax.swing.Action[]{addAction});

                    // creatings section panels for Chapters
                    SectionPanel[] pan = new SectionPanel[chapters.length];
                    for (int i = 0; i < chapters.length; i++) {
                        pan[i] = new SectionPanel(this, chapterNode[i], chapters[i]);
                        //pan[i].setHeaderActions(new javax.swing.Action[]{removeAction});
                        chaptersCont.addSection(pan[i]);
                    }
                    addSection(chaptersCont);
                }

                // ADL Components
                {
                    Component[] links = book.getComponents().getComponent();
                    Node[] chapterNode = new Node[links.length];
                    Children ch = new Children.Array();
                    for (int i = 0; i < links.length; i++) {
                        chapterNode[i] = new ComponentNode(links[i]);
                    }
                    ch.add(chapterNode);

                    Node chaptersNode = new SectionContainerNode(ch);
                    chaptersNode.setDisplayName("Components");
                    rootChildren.add(new Node[]{chaptersNode});

                    // Topology Nodes Section
                    SectionContainer linksCont = new SectionContainer(this, chaptersNode, "Components");
                    //linksCont.setHeaderActions(new javax.swing.Action[]{addAction});

                    // creatings section panels for Topology Nodes
                    SectionPanel[] pan = new SectionPanel[links.length];
                    for (int i = 0; i < links.length; i++) {
                        pan[i] = new SectionPanel(this, chapterNode[i], links[i]);
                        //pan[i].setHeaderActions(new javax.swing.Action[]{removeAction});
                        linksCont.addSection(pan[i]);
                    }
                    addSection(linksCont);
                }

                // ADL Channels
                {
                    Channel[] links = book.getChannels().getChannel();
                    Node[] chapterNode = new Node[links.length];
                    Children ch = new Children.Array();
                    for (int i = 0; i < links.length; i++) {
                        chapterNode[i] = new ChannelNode(links[i]);
                    }
                    ch.add(chapterNode);

                    Node chaptersNode = new SectionContainerNode(ch);
                    chaptersNode.setDisplayName("Channels");
                    rootChildren.add(new Node[]{chaptersNode});

                    // Topology Nodes Section
                    SectionContainer linksCont = new SectionContainer(this, chaptersNode, "Channels");
                    //linksCont.setHeaderActions(new javax.swing.Action[]{addAction});

                    // creatings section panels for Topology Nodes
                    SectionPanel[] pan = new SectionPanel[links.length];
                    for (int i = 0; i < links.length; i++) {
                        pan[i] = new SectionPanel(this, chapterNode[i], links[i]);
                        //pan[i].setHeaderActions(new javax.swing.Action[]{removeAction});
                        linksCont.addSection(pan[i]);
                    }
                    addSection(linksCont);
                }

                // ADL Bindings
                {
                    Binding[] links = book.getBindings().getBinding();
                    Node[] chapterNode = new Node[links.length];
                    Children ch = new Children.Array();
                    for (int i = 0; i < links.length; i++) {
                        chapterNode[i] = new BindingNode(links[i]);
                    }
                    ch.add(chapterNode);

                    Node chaptersNode = new SectionContainerNode(ch);
                    chaptersNode.setDisplayName("Bindings");
                    rootChildren.add(new Node[]{chaptersNode});

                    // Topology Nodes Section
                    SectionContainer linksCont = new SectionContainer(this, chaptersNode, "Bindings");
                    //linksCont.setHeaderActions(new javax.swing.Action[]{addAction});

                    // creatings section panels for Topology Nodes
                    SectionPanel[] pan = new SectionPanel[links.length];
                    for (int i = 0; i < links.length; i++) {
                        pan[i] = new SectionPanel(this, chapterNode[i], links[i]);
                        //pan[i].setHeaderActions(new javax.swing.Action[]{removeAction});
                        linksCont.addSection(pan[i]);
                    }
                    addSection(linksCont);
                }

                // ADL Subscriptions
                {
                    Subscription[] links = book.getSubscriptions().getSubscription();
                    Node[] chapterNode = new Node[links.length];
                    Children ch = new Children.Array();
                    for (int i = 0; i < links.length; i++) {
                        chapterNode[i] = new SubscriptionNode(links[i]);
                    }
                    ch.add(chapterNode);

                    Node chaptersNode = new SectionContainerNode(ch);
                    chaptersNode.setDisplayName("Subscriptions");
                    rootChildren.add(new Node[]{chaptersNode});

                    // Topology Nodes Section
                    SectionContainer linksCont = new SectionContainer(this, chaptersNode, "Subscriptions");
                    //linksCont.setHeaderActions(new javax.swing.Action[]{addAction});

                    // creatings section panels for Topology Nodes
                    SectionPanel[] pan = new SectionPanel[links.length];
                    for (int i = 0; i < links.length; i++) {
                        pan[i] = new SectionPanel(this, chapterNode[i], links[i]);
                        //pan[i].setHeaderActions(new javax.swing.Action[]{removeAction});
                        linksCont.addSection(pan[i]);
                    }
                    addSection(linksCont);
                }

                // Startup sequence
                {
                    Node[] chapterNode = new Node[2];
                    Children ch = new Children.Array();
                    chapterNode[0] = new StartComponentsNode(book.getStartSequence().getStartComponents());
                    chapterNode[1] = new StartChannelsNode(book.getStartSequence().getStartChannels());
                    ch.add(chapterNode);

                    Node chaptersNode = new SectionContainerNode(ch);
                    chaptersNode.setDisplayName("Start sequences");
                    rootChildren.add(new Node[]{chaptersNode});

                    // Topology Nodes Section
                    SectionContainer linksCont = new SectionContainer(this, chaptersNode, "Start sequences");
                    //linksCont.setHeaderActions(new javax.swing.Action[]{addAction});

                    // creatings section panels for Topology Nodes
                    SectionPanel[] pan = new SectionPanel[2];
                    pan[0] = new SectionPanel(this, chapterNode[0], book.getStartSequence().getStartComponents());
                    linksCont.addSection(pan[0]);
                    pan[1] = new SectionPanel(this, chapterNode[1], book.getStartSequence().getStartChannels());
                    linksCont.addSection(pan[1]);

                    addSection(linksCont);
                }

            } catch (java.io.IOException ex) {
                System.out.println("ex=" + ex);
                root.setDisplayName("Invalid Configuration");
            }
            setRoot(root);
        }

        @Override
        public org.netbeans.modules.xml.multiview.Error validateView() {
            try {
                Configuration book = dataObject.getConfiguration();
                String title = "Configurations tag..."; //book.getTitle();
                if (title == null || title.length() == 0) {
                    org.netbeans.modules.xml.multiview.Error.ErrorLocation loc = new ErrorLocation(book, "title"); //NOI18N
                    return new org.netbeans.modules.xml.multiview.Error(org.netbeans.modules.xml.multiview.Error.MISSING_VALUE_MESSAGE, "Title", loc);
                }
                /* TODO Chapter[] chapters = book.getChapter();
                for (int i = 0; i < chapters.length; i++) {
                title = chapters[i].getTitle();
                if (title == null || title.length() == 0) {
                Error.ErrorLocation loc = new Error.ErrorLocation(chapters[i], "title");
                return new Error(Error.MISSING_VALUE_MESSAGE, "Title", loc);
                }
                for (int j = 0; j < chapters.length; j++) {
                String tit = chapters[j].getTitle();
                if (i != j && title.equals(tit)) {
                Error.ErrorLocation loc = new Error.ErrorLocation(chapters[i], "title");
                return new Error(Error.TYPE_FATAL, Error.DUPLICATE_VALUE_MESSAGE, title, loc);
                }
                }
                }*/
            } catch (java.io.IOException ex) {
            }
            return null;
        }
    }

    private class ConfigurationNode extends org.openide.nodes.AbstractNode {

        ConfigurationNode(Configuration book) {
            super(org.openide.nodes.Children.LEAF);
            setDisplayName("TBN Configuration");
        //setIconBase("org/netbeans/modules/web/dd/multiview/resources/class"); //NOI18N
        }
    }

    private class FactoryNode extends org.openide.nodes.AbstractNode {

        FactoryNode(Factory chapter) {
            super(org.openide.nodes.Children.LEAF);
            setDisplayName(chapter.getFactoryName());
        //setIconBase("org/netbeans/modules/bookmultiview/Datasource.gif"); //NOI18N
        }
    }

    private class ComponentNode extends org.openide.nodes.AbstractNode {

        ComponentNode(Component chapter) {
            super(org.openide.nodes.Children.LEAF);
            setDisplayName(chapter.getComponentName());
        //setIconBase("org/netbeans/modules/bookmultiview/Datasource.gif"); //NOI18N
        }
    }

    private class ChannelNode extends org.openide.nodes.AbstractNode {

        ChannelNode(Channel chapter) {
            super(org.openide.nodes.Children.LEAF);
            setDisplayName(chapter.getChannelName());
        //setIconBase("org/netbeans/modules/bookmultiview/Datasource.gif"); //NOI18N
        }
    }

    private class BindingNode extends org.openide.nodes.AbstractNode {

        BindingNode(Binding chapter) {
            super(org.openide.nodes.Children.LEAF);
            setDisplayName(chapter.getComponentName() + " --> " + chapter.getChannelName());
        //setIconBase("org/netbeans/modules/bookmultiview/Datasource.gif"); //NOI18N
        }
    }

    private class SubscriptionNode extends org.openide.nodes.AbstractNode {

        SubscriptionNode(Subscription chapter) {
            super(org.openide.nodes.Children.LEAF);
            setDisplayName(chapter.getComponentName() + " <-- " + chapter.getChannelName());
        //setIconBase("org/netbeans/modules/bookmultiview/Datasource.gif"); //NOI18N
        }
    }

    private class StartComponentsNode extends org.openide.nodes.AbstractNode {

        StartComponentsNode(StartComponents chapter) {
            super(org.openide.nodes.Children.LEAF);
            setDisplayName("Components");
        //setIconBase("org/netbeans/modules/bookmultiview/Datasource.gif"); //NOI18N
        }
    }

    private class StartChannelsNode extends org.openide.nodes.AbstractNode {

        StartChannelsNode(StartChannels chapter) {
            super(org.openide.nodes.Children.LEAF);
            setDisplayName("Channels");
        //setIconBase("org/netbeans/modules/bookmultiview/Datasource.gif"); //NOI18N
        }
    }
}