/*
 * PanelFactory.java
 * 
 * 
 * Copyright (C) 2008 Markus Kilås
 *  
 * This file is part of TBN NetBeans Module.
 *
 * TBN NetBeans Module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TBN NetBeans Module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TBN NetBeans Module.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package com.markuspage.markus.tbn.module1.adl;

import com.markuspage.markus.tbn.module1.adl.model.Binding;
import com.markuspage.markus.tbn.module1.adl.model.Channel;
import com.markuspage.markus.tbn.module1.adl.model.Component;
import com.markuspage.markus.tbn.module1.adl.model.Configuration;
import com.markuspage.markus.tbn.module1.adl.model.Factory;
import com.markuspage.markus.tbn.module1.adl.model.StartChannels;
import com.markuspage.markus.tbn.module1.adl.model.StartComponents;
import com.markuspage.markus.tbn.module1.adl.model.Subscription;
import org.netbeans.modules.xml.multiview.ui.InnerPanelFactory;
import org.netbeans.modules.xml.multiview.ui.SectionInnerPanel;
import org.netbeans.modules.xml.multiview.ui.SectionView;
import org.netbeans.modules.xml.multiview.ui.ToolBarDesignEditor;

/**
 *
 * @author markus
 */
public class PanelFactory implements InnerPanelFactory {

    private ADLDataObject dObj;
    private ToolBarDesignEditor editor;

    public PanelFactory(ToolBarDesignEditor editor, ADLDataObject dObj) {
        this.dObj = dObj;
        this.editor = editor;
    }

    public SectionInnerPanel createInnerPanel(Object key) {
        if (key instanceof Configuration) {
            return new ADLPanel((SectionView) editor.getContentView(), dObj, (Configuration) key);
        } else if (key instanceof Factory) {
            return new FactoryPanel((SectionView) editor.getContentView(), dObj, (Factory) key);
        } else if (key instanceof Component) {
            return new ComponentPanel((SectionView) editor.getContentView(), dObj, (Component) key);
        } else if (key instanceof Channel) {
            return new ChannelPanel((SectionView) editor.getContentView(), dObj, (Channel) key);
        } else if (key instanceof Binding) {
            return new BindingPanel((SectionView) editor.getContentView(), dObj, (Binding) key);
        } else if (key instanceof Subscription) {
            return new SubscriptionPanel((SectionView) editor.getContentView(), dObj, (Subscription) key);
        } else if (key instanceof StartComponents) {
            return new StartComponentsPanel((SectionView) editor.getContentView(), dObj, (StartComponents) key);
        } else if (key instanceof StartChannels) {
            return new StartChannelsPanel((SectionView) editor.getContentView(), dObj, (StartChannels) key);
        } else {
            throw new RuntimeException("No panel for type "+key.getClass());
        }
    }
}
