/*
 * Code3WizardPanel2.java
 * 
 * 
 * Copyright (C) 2008 Markus Kilås
 *  
 * This file is part of TBN NetBeans Module.
 *
 * TBN NetBeans Module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TBN NetBeans Module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TBN NetBeans Module.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package com.markuspage.markus.tbn.module1.code3;

import com.markuspage.markus.tbn.module1.adl.ADLDataObject;
import com.markuspage.markus.tbn.module1.adl.model.Channel;
import com.markuspage.markus.tbn.module1.adl.model.Factory;
import com.markuspage.markus.tbn.module1.adl.model.Component;
import com.markuspage.markus.tbn.module1.adl.model.Subscription;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import javax.swing.event.ChangeListener;
import org.openide.WizardDescriptor;
import org.openide.util.Exceptions;
import org.openide.util.HelpCtx;

public class Code3WizardPanel2 implements WizardDescriptor.Panel {

    /**
     * The visual component that displays this panel. If you need to access the
     * component from this class, just use getComponent().
     */
    private java.awt.Component component;
    
    private ArrayList<ClassTemplate> newClasses = new ArrayList<ClassTemplate>();
    
   
    // Get the visual component for the panel. In this template, the component
    // is kept separate. This can be more efficient: if the wizard is created
    // but never displayed, or not all panels are displayed, it is better to
    // create only those which really need to be visible.
    public java.awt.Component getComponent() {
        if (component == null) {
            component = new Code3VisualPanel2();
        }
        return component;
    }

    public HelpCtx getHelp() {
        // Show no Help button for this panel:
        return HelpCtx.DEFAULT_HELP;
    // If you have context help:
    // return new HelpCtx(SampleWizardPanel1.class);
    }

    public boolean isValid() {
        // If it is always OK to press Next or Finish, then:
        return true;
    // If it depends on some condition (form filled out...), then:
    // return someCondition();
    // and when this condition changes (last form field filled in...) then:
    // fireChangeEvent();
    // and uncomment the complicated stuff below.
    }

    public final void addChangeListener(ChangeListener l) {
    }

    public final void removeChangeListener(ChangeListener l) {
    }
    /*
    private final Set<ChangeListener> listeners = new HashSet<ChangeListener>(1); // or can use ChangeSupport in NB 6.0
    public final void addChangeListener(ChangeListener l) {
    synchronized (listeners) {
    listeners.add(l);
    }
    }
    public final void removeChangeListener(ChangeListener l) {
    synchronized (listeners) {
    listeners.remove(l);
    }
    }
    protected final void fireChangeEvent() {
    Iterator<ChangeListener> it;
    synchronized (listeners) {
    it = new HashSet<ChangeListener>(listeners).iterator();
    }
    ChangeEvent ev = new ChangeEvent(this);
    while (it.hasNext()) {
    it.next().stateChanged(ev);
    }
    }
     */

    // You can use a settings object to keep track of state. Normally the
    // settings object will be the WizardDescriptor, so you can use
    // WizardDescriptor.getProperty & putProperty to store information entered
    // by the user.
    public void readSettings(Object settings) {
        
        ArrayList<ADLDataObject> adlObjects = (ArrayList<ADLDataObject>)((WizardDescriptor) settings).getProperty(Code3VisualPanel1.ADLOBJECTS);
        generateWhatToDo(adlObjects);
    }

    public void storeSettings(Object settings) {
        System.out.println("Panel2.readSettings");
        ((WizardDescriptor)settings).putProperty(Code3VisualPanel2.NEWCLASSES, newClasses);
    }

    private ClassTemplate findClass(Collection<? extends ClassTemplate> newClasses, String className, String packagePath) {
        for(ClassTemplate clazz : newClasses) {
            if(className.equals(clazz.getProperties().get("name")) && packagePath.equals(clazz.getProperties().get("package"))) {
                return clazz;
            }
        }
        return null;
    }

    private Component findComponentByFactoryName(ADLDataObject adl, String factoryName) throws IOException {
        for(Component c : adl.getConfiguration().getComponents().getComponent()) {
            if(factoryName.equals(c.getFactoryName())) {
                return c;
            }
        }
        return null;
    }

    private void generateWhatToDo(ArrayList<ADLDataObject> adlObjects) {
        newClasses.clear();
        
        for(ADLDataObject adl : adlObjects) {
            try {
                // Events from channels
                for (Channel channel : adl.getConfiguration().getChannels().getChannel()) {
                    for(String eventClass : channel.getEventClass()) {
                        String className = eventClass.substring(eventClass.lastIndexOf('.')+1);
                        String packagePath = eventClass.substring(0, eventClass.lastIndexOf('.'));
                        if(findClass(newClasses, className, packagePath) == null) {
                            newClasses.add(new ClassTemplate(className+".java", "Event", className, packagePath));
                        }
                    }
                }
                
                // Components
                for(Factory factory : adl.getConfiguration().getFactories().getFactory()) {
                    
                    String componentClass = factory.getComponentClass();
                    String className = componentClass.substring(componentClass.lastIndexOf('.')+1);
                    String packagePath = componentClass.substring(0, componentClass.lastIndexOf('.'));
                    
                    ClassTemplate clazz = findClass(newClasses, className, packagePath);
                    if(clazz == null) {
                        clazz = new ClassTemplate(className+".java", "Component", className, packagePath);
                        newClasses.add(clazz);
                    }
                    //clazz.getProperties().put("factoryName", factory.getFactoryName());
                    
                    String factoryName = factory.getFactoryName();
                    
                    Component c = findComponentByFactoryName(adl, factoryName);
                    String fullComponentName = factoryName + ":" + c.getComponentName();
                    
                    // Subscriptions
                    for(Subscription subscription : adl.getConfiguration().getSubscriptions().getSubscription()) {
                        if(fullComponentName.equals(subscription.getComponentName())) {
                            String handlerName = subscription.getHandlerName();
                            String eventName = handlerName.substring("handle".length());
                            clazz.addMethod("\tpublic void "+handlerName+"("+eventName+" event"+") {\n\t\t//TODO: Add implementation here\n\t}");
                        }
                    }
                }
                
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }
        }
        
        // Save result
        ((Code3VisualPanel2)getComponent()).setNewClasses(newClasses);
    }

}

