/*
 * TopologyToolbarMVElement.java
 * 
 * 
 * Copyright (C) 2008 Markus Kilås
 *  
 * This file is part of TBN NetBeans Module.
 *
 * TBN NetBeans Module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TBN NetBeans Module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TBN NetBeans Module.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package com.markuspage.markus.tbn.module1.topology;

import com.markuspage.markus.tbn.module1.topology.model.Topology;
import org.netbeans.modules.xml.multiview.Error.ErrorLocation;
import org.netbeans.modules.xml.multiview.ToolBarMultiViewElement;
import org.netbeans.modules.xml.multiview.ui.SectionContainer;
import org.netbeans.modules.xml.multiview.ui.SectionContainerNode;
import org.netbeans.modules.xml.multiview.ui.SectionPanel;
import org.netbeans.modules.xml.multiview.ui.SectionView;
import org.netbeans.modules.xml.multiview.ui.ToolBarDesignEditor;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;

/**
 *
 * @author markus
 */
public class TopologyToolBarMVElement extends ToolBarMultiViewElement {

    private ToolBarDesignEditor comp;
    private SectionView view;
    private TopologyDataObject dataObject;
    private PanelFactory factory;

    public TopologyToolBarMVElement(TopologyDataObject dObj) {
        super(dObj);
        this.dataObject = dObj;
        comp = new ToolBarDesignEditor();
        factory = new PanelFactory(comp, dObj);
        setVisualEditor(comp);
    }

    @Override
    public SectionView getSectionView() {
        return view;
    }

    @Override
    public void componentShowing() {
        super.componentShowing();
        view = new TopologyView(dataObject);
        comp.setContentView(view);
        try {
            view.openPanel(dataObject.getTopology());
        } catch (java.io.IOException ex) {
        }
        view.checkValidity();
    }

    private class TopologyView extends SectionView {

        TopologyView(TopologyDataObject dObj) {
            super(factory);

            Children rootChildren = new Children.Array();
            Node root = new AbstractNode(rootChildren);
            try {
                Topology book = dObj.getTopology();
                Node bookNode = new TopologyNode(book);

                // add panels
                addSection(new SectionPanel(this, bookNode, book)); //NOI18N


                // Topology Nodes
                {
                    com.markuspage.markus.tbn.module1.topology.model.Node[] chapters = book.getNodes().getNode();
                    Node[] chapterNode = new Node[chapters.length];
                    Children ch = new Children.Array();
                    for (int i = 0; i < chapters.length; i++) {
                        chapterNode[i] = new NodeNode(chapters[i]);
                    }
                    ch.add(chapterNode);

                    Node chaptersNode = new SectionContainerNode(ch);
                    chaptersNode.setDisplayName("Nodes");
                    rootChildren.add(new Node[]{bookNode, chaptersNode});


                    // Topology Nodes Section
                    SectionContainer chaptersCont = new SectionContainer(this, chaptersNode, "Nodes");
                    //jspPGCont.setHeaderActions(new javax.swing.Action[]{addAction});

                    // creatings section panels for Topology Nodes
                    SectionPanel[] pan = new SectionPanel[chapters.length];
                    for (int i = 0; i < chapters.length; i++) {
                        pan[i] = new SectionPanel(this, chapterNode[i], chapters[i]);
                        //pan[i].setHeaderActions(new javax.swing.Action[]{removeAction});
                        chaptersCont.addSection(pan[i]);
                    }
                    addSection(chaptersCont);
                }

                // Topology Links
                {
                    com.markuspage.markus.tbn.module1.topology.model.Link[] links = book.getLinks().getLink();
                    Node[] chapterNode = new Node[links.length];
                    Children ch = new Children.Array();
                    for (int i = 0; i < links.length; i++) {
                        chapterNode[i] = new LinkNode(links[i]);
                    }
                    ch.add(chapterNode);

                    Node chaptersNode = new SectionContainerNode(ch);
                    chaptersNode.setDisplayName("Links");
                    rootChildren.add(new Node[]{bookNode, chaptersNode});


                    // Topology Nodes Section
                    SectionContainer linksCont = new SectionContainer(this, chaptersNode, "Links");
                    //linksCont.setHeaderActions(new javax.swing.Action[]{addAction});

                    // creatings section panels for Topology Nodes
                    SectionPanel[] pan = new SectionPanel[links.length];
                    for (int i = 0; i < links.length; i++) {
                        pan[i] = new SectionPanel(this, chapterNode[i], links[i]);
                        //pan[i].setHeaderActions(new javax.swing.Action[]{removeAction});
                        linksCont.addSection(pan[i]);
                    }
                    addSection(linksCont);
                }

            } catch (java.io.IOException ex) {
                System.out.println("ex=" + ex);
                root.setDisplayName("Invalid Book");
            }
            setRoot(root);
        }

        @Override
        public org.netbeans.modules.xml.multiview.Error validateView() {
            try {
                Topology book = dataObject.getTopology();
                String title = "Configurations tag..."; //book.getTitle();
                if (title == null || title.length() == 0) {
                    org.netbeans.modules.xml.multiview.Error.ErrorLocation loc = new ErrorLocation(book, "title"); //NOI18N
                    return new org.netbeans.modules.xml.multiview.Error(org.netbeans.modules.xml.multiview.Error.MISSING_VALUE_MESSAGE, "Title", loc);
                }
                /* TODO Chapter[] chapters = book.getChapter();
                for (int i = 0; i < chapters.length; i++) {
                title = chapters[i].getTitle();
                if (title == null || title.length() == 0) {
                Error.ErrorLocation loc = new Error.ErrorLocation(chapters[i], "title");
                return new Error(Error.MISSING_VALUE_MESSAGE, "Title", loc);
                }
                for (int j = 0; j < chapters.length; j++) {
                String tit = chapters[j].getTitle();
                if (i != j && title.equals(tit)) {
                Error.ErrorLocation loc = new Error.ErrorLocation(chapters[i], "title");
                return new Error(Error.TYPE_FATAL, Error.DUPLICATE_VALUE_MESSAGE, title, loc);
                }
                }
                }*/
            } catch (java.io.IOException ex) {
            }
            return null;
        }
    }

    private class TopologyNode extends org.openide.nodes.AbstractNode {

        TopologyNode(Topology book) {
            super(org.openide.nodes.Children.LEAF);
            setDisplayName("Topology-Node...");
        //setIconBase("org/netbeans/modules/web/dd/multiview/resources/class"); //NOI18N
        }
    }

    private class NodeNode extends org.openide.nodes.AbstractNode {

        NodeNode(com.markuspage.markus.tbn.module1.topology.model.Node chapter) {
            super(org.openide.nodes.Children.LEAF);
            setDisplayName("Node " + chapter.getAttributeValue("id"));
            setIconBaseWithExtension("org/netbeans/modules/bookmultiview/Datasource.gif"); //NOI18N
        }
    }

    private class LinkNode extends org.openide.nodes.AbstractNode {

        LinkNode(com.markuspage.markus.tbn.module1.topology.model.Link chapter) {
            super(org.openide.nodes.Children.LEAF);

            String dir;
            if ("true".equals(chapter.getAttributeValue("undirected"))) {
                dir = " <--> ";
            } else {
                dir = " --> ";
            }

            setDisplayName("Link " + chapter.getAttributeValue("src_id") + dir + chapter.getAttributeValue("dst_id"));
            setIconBaseWithExtension("org/netbeans/modules/bookmultiview/Datasource.gif"); //NOI18N
        }
    }
}


